@@ -9,6 +9,7 @@ mod middleware;
 mod nested_json;
 mod netrc;
 mod printer;
+mod redacted;
 mod redirect;
 mod request_items;
 mod session;
@@ -19,7 +20,7 @@ mod vendored;
 use std::env;
 use std::fs::File;
 use std::io::{self, IsTerminal, Read};
-use std::net::{IpAddr, SocketAddr};
+use std::net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr};
 use std::path::PathBuf;
 use std::process;
 use std::str::FromStr;
@@ -34,6 +35,7 @@ use reqwest::header::{
 };
 use reqwest::tls;
 use url::Host;
+use utils::reason_phrase;
 
 use crate::auth::{Auth, DigestAuthMiddleware};
 use crate::buffer::Buffer;
@@ -60,7 +62,16 @@ fn get_user_agent() -> &'static str {
 
 fn main() {
     let args = Cli::parse();
-    let bin_name = args.bin_name.clone();
+
+    if args.debug {
+        setup_backtraces();
+    }
+    args.logger_config().init();
+    // HTTPie also prints the language version, library versions, and OS version.
+    // But those are harder to access for us (and perhaps less likely to cause quirks).
+    log::debug!("xh {} {}", env!("CARGO_PKG_VERSION"), env!("XH_FEATURES"));
+    log::debug!("{args:#?}");
+
     let native_tls = args.native_tls;
 
     match run(args) {
@@ -68,7 +79,8 @@ fn main() {
             process::exit(exit_code);
         }
         Err(err) => {
-            eprintln!("{}: error: {:?}", bin_name, err);
+            log::debug!("{err:#?}");
+            log::error!("{err:?}");
             let msg = err.root_cause().to_string();
             if native_tls && msg == "invalid minimum TLS version for backend" {
                 eprintln!();
@@ -93,13 +105,9 @@ fn run(args: Cli) -> Result<i32> {
         return Ok(0);
     }
 
-    let warn = {
-        let bin_name = &args.bin_name;
-        move |msg| eprintln!("{}: warning: {}", bin_name, msg)
-    };
-
     let (mut headers, headers_to_unset) = args.request_items.headers()?;
     let url = url_with_query(args.url, &args.request_items.query()?);
+    log::debug!("Complete URL: {url}");
 
     let use_stdin = !(args.ignore_stdin || io::stdin().is_terminal() || test_pretend_term());
 
@@ -131,6 +139,7 @@ fn run(args: Cli) -> Result<i32> {
     };
 
     let method = args.method.unwrap_or_else(|| body.pick_method());
+    log::debug!("HTTP method: {method}");
 
     let mut client = Client::builder()
         .http1_title_case_headers()
@@ -153,13 +162,13 @@ fn run(args: Cli) -> Result<i32> {
 
         #[cfg(feature = "native-tls")]
         if !args.native_tls && tls_version < tls::Version::TLS_1_2 {
-            warn("rustls does not support older TLS versions. native-tls will be enabled. Use --native-tls to silence this warning.");
+            log::warn!("rustls does not support older TLS versions. native-tls will be enabled. Use --native-tls to silence this warning.");
             client = client.use_native_tls();
         }
 
         #[cfg(not(feature = "native-tls"))]
         if tls_version < tls::Version::TLS_1_2 {
-            warn("rustls does not support older TLS versions. Consider building with the `native-tls` feature enabled.");
+            log::warn!("rustls does not support older TLS versions. Consider building with the `native-tls` feature enabled.");
         }
     }
 
@@ -178,85 +187,83 @@ fn run(args: Cli) -> Result<i32> {
     let mut auth = None;
     let mut save_auth_in_session = true;
 
-    if url.scheme() == "https" {
-        let verify = args.verify.unwrap_or_else(|| {
-            // requests library which is used by HTTPie checks for both
-            // REQUESTS_CA_BUNDLE and CURL_CA_BUNDLE environment variables.
-            // See https://docs.python-requests.org/en/master/user/advanced/#ssl-cert-verification
-            if let Some(path) = env::var_os("REQUESTS_CA_BUNDLE") {
-                Verify::CustomCaBundle(PathBuf::from(path))
-            } else if let Some(path) = env::var_os("CURL_CA_BUNDLE") {
-                Verify::CustomCaBundle(PathBuf::from(path))
-            } else {
-                Verify::Yes
-            }
-        });
-        client = match verify {
-            Verify::Yes => client,
-            Verify::No => client.danger_accept_invalid_certs(true),
-            Verify::CustomCaBundle(path) => {
-                if args.native_tls {
-                    // This is not a hard error in case it gets fixed upstream
-                    // https://github.com/seanmonstar/reqwest/issues/1260
-                    warn("Custom CA bundles with native-tls are broken");
-                }
-
-                let mut buffer = Vec::new();
-                let mut file = File::open(&path).with_context(|| {
-                    format!("Failed to open the custom CA bundle: {}", path.display())
-                })?;
-                file.read_to_end(&mut buffer).with_context(|| {
-                    format!("Failed to read the custom CA bundle: {}", path.display())
-                })?;
-
-                client = client.tls_built_in_root_certs(false);
-                for pem in pem::parse_many(buffer)? {
-                    let certificate = reqwest::Certificate::from_pem(pem::encode(&pem).as_bytes())
-                        .with_context(|| {
-                            format!("Failed to load the custom CA bundle: {}", path.display())
-                        })?;
-                    client = client.add_root_certificate(certificate);
-                }
-                client
-            }
-        };
-
-        #[cfg(feature = "rustls")]
-        if let Some(cert) = args.cert {
+    let verify = args.verify.unwrap_or_else(|| {
+        // requests library which is used by HTTPie checks for both
+        // REQUESTS_CA_BUNDLE and CURL_CA_BUNDLE environment variables.
+        // See https://docs.python-requests.org/en/master/user/advanced/#ssl-cert-verification
+        if let Some(path) = env::var_os("REQUESTS_CA_BUNDLE") {
+            Verify::CustomCaBundle(PathBuf::from(path))
+        } else if let Some(path) = env::var_os("CURL_CA_BUNDLE") {
+            Verify::CustomCaBundle(PathBuf::from(path))
+        } else {
+            Verify::Yes
+        }
+    });
+    client = match verify {
+        Verify::Yes => client,
+        Verify::No => client.danger_accept_invalid_certs(true),
+        Verify::CustomCaBundle(path) => {
             if args.native_tls {
-                // Unlike the --verify case this is advertised to not work, so it's
-                // not an outright bug, but it's still imaginable that it'll start working
-                warn("Client certificates are not supported for native-tls");
+                // This is not a hard error in case it gets fixed upstream
+                // https://github.com/seanmonstar/reqwest/issues/1260
+                log::warn!("Custom CA bundles with native-tls are broken");
             }
 
             let mut buffer = Vec::new();
-            let mut file = File::open(&cert)
-                .with_context(|| format!("Failed to open the cert file: {}", cert.display()))?;
-            file.read_to_end(&mut buffer)
-                .with_context(|| format!("Failed to read the cert file: {}", cert.display()))?;
+            let mut file = File::open(&path).with_context(|| {
+                format!("Failed to open the custom CA bundle: {}", path.display())
+            })?;
+            file.read_to_end(&mut buffer).with_context(|| {
+                format!("Failed to read the custom CA bundle: {}", path.display())
+            })?;
 
-            if let Some(cert_key) = args.cert_key {
-                buffer.push(b'\n');
-
-                let mut file = File::open(&cert_key).with_context(|| {
-                    format!("Failed to open the cert key file: {}", cert_key.display())
-                })?;
-                file.read_to_end(&mut buffer).with_context(|| {
-                    format!("Failed to read the cert key file: {}", cert_key.display())
-                })?;
+            client = client.tls_built_in_root_certs(false);
+            for pem in pem::parse_many(buffer)? {
+                let certificate = reqwest::Certificate::from_pem(pem::encode(&pem).as_bytes())
+                    .with_context(|| {
+                        format!("Failed to load the custom CA bundle: {}", path.display())
+                    })?;
+                client = client.add_root_certificate(certificate);
             }
+            client
+        }
+    };
 
-            // We may fail here if we can't parse it but also if we don't have the key
-            let identity = reqwest::Identity::from_pem(&buffer)
-                .context("Failed to load the cert/cert key files")?;
-            client = client.identity(identity);
-        };
-        #[cfg(not(feature = "rustls"))]
-        if args.cert.is_some() {
+    #[cfg(feature = "rustls")]
+    if let Some(cert) = args.cert {
+        if args.native_tls {
             // Unlike the --verify case this is advertised to not work, so it's
             // not an outright bug, but it's still imaginable that it'll start working
-            warn("Client certificates are not supported for native-tls and this binary was built without rustls support");
-        };
+            log::warn!("Client certificates are not supported for native-tls");
+        }
+
+        let mut buffer = Vec::new();
+        let mut file = File::open(&cert)
+            .with_context(|| format!("Failed to open the cert file: {}", cert.display()))?;
+        file.read_to_end(&mut buffer)
+            .with_context(|| format!("Failed to read the cert file: {}", cert.display()))?;
+
+        if let Some(cert_key) = args.cert_key {
+            buffer.push(b'\n');
+
+            let mut file = File::open(&cert_key).with_context(|| {
+                format!("Failed to open the cert key file: {}", cert_key.display())
+            })?;
+            file.read_to_end(&mut buffer).with_context(|| {
+                format!("Failed to read the cert key file: {}", cert_key.display())
+            })?;
+        }
+
+        // We may fail here if we can't parse it but also if we don't have the key
+        let identity = reqwest::Identity::from_pem(&buffer)
+            .context("Failed to load the cert/cert key files")?;
+        client = client.identity(identity);
+    }
+    #[cfg(not(feature = "rustls"))]
+    if args.cert.is_some() {
+        // Unlike the --verify case this is advertised to not work, so it's
+        // not an outright bug, but it's still imaginable that it'll start working
+        log::warn!("Client certificates are not supported for native-tls and this binary was built without rustls support");
     }
 
     for proxy in args.proxy.into_iter().rev() {
@@ -267,23 +274,19 @@ fn run(args: Cli) -> Result<i32> {
         }?);
     }
 
-    if matches!(
-        args.http_version,
-        Some(HttpVersion::Http10) | Some(HttpVersion::Http11)
-    ) {
-        client = client.http1_only();
-    }
-
-    if matches!(args.http_version, Some(HttpVersion::Http2PriorKnowledge)) {
-        client = client.http2_prior_knowledge();
-    }
+    client = match args.http_version {
+        Some(HttpVersion::Http10 | HttpVersion::Http11) => client.http1_only(),
+        Some(HttpVersion::Http2PriorKnowledge) => client.http2_prior_knowledge(),
+        Some(HttpVersion::Http2) => client,
+        None => client,
+    };
 
     let cookie_jar = Arc::new(reqwest_cookie_store::CookieStoreMutex::default());
     client = client.cookie_provider(cookie_jar.clone());
 
     client = match (args.ipv4, args.ipv6) {
-        (true, false) => client.local_address(IpAddr::from_str("0.0.0.0")?),
-        (false, true) => client.local_address(IpAddr::from_str("::")?),
+        (true, false) => client.local_address(IpAddr::from(Ipv4Addr::UNSPECIFIED)),
+        (false, true) => client.local_address(IpAddr::from(Ipv6Addr::UNSPECIFIED)),
         _ => client,
     };
 
@@ -317,6 +320,7 @@ fn run(args: Cli) -> Result<i32> {
                             None
                         })
                         .with_context(|| format!("Couldn't bind to {:?}", name_or_ip))?;
+                    log::debug!("Resolved {name_or_ip:?} to {ip_addr:?}");
                     client = client.local_address(ip_addr);
                 }
             }
@@ -327,6 +331,8 @@ fn run(args: Cli) -> Result<i32> {
         client = client.resolve(&resolve.domain, SocketAddr::new(resolve.addr, 0));
     }
 
+    log::trace!("Finalizing reqwest client");
+    log::trace!("{client:#?}");
     let client = client.build()?;
 
     let mut session = match &args.session {
@@ -365,7 +371,7 @@ fn run(args: Cli) -> Result<i32> {
             .request(method, url.clone())
             .header(
                 ACCEPT_ENCODING,
-                HeaderValue::from_static("gzip, deflate, br"),
+                HeaderValue::from_static("gzip, deflate, br, zstd"),
             )
             .header(USER_AGENT, get_user_agent());
 
@@ -420,14 +426,21 @@ fn run(args: Cli) -> Result<i32> {
             Body::File {
                 file_name,
                 file_type,
-                // We could turn this into a Content-Disposition header, but
-                // that has no effect, so just ignore it
-                // (Additional precedent: HTTPie ignores file_type here)
-                file_name_header: _,
-            } => request_builder.body(File::open(file_name)?).header(
-                CONTENT_TYPE,
-                file_type.unwrap_or_else(|| HeaderValue::from_static(JSON_CONTENT_TYPE)),
-            ),
+                file_name_header,
+            } => {
+                if file_name_header.is_some() {
+                    // Content-Disposition headers aren't allowed in this context (only responses
+                    // and multipart request parts), so just ignore it
+                    // (Additional precedent: HTTPie ignores file_type here)
+                    log::warn!(
+                        "Ignoring ;filename= tag for single-file body. Consider --multipart."
+                    );
+                }
+                request_builder.body(File::open(file_name)?).header(
+                    CONTENT_TYPE,
+                    file_type.unwrap_or_else(|| HeaderValue::from_static(JSON_CONTENT_TYPE)),
+                )
+            }
         };
 
         if args.resume {
@@ -484,6 +497,10 @@ fn run(args: Cli) -> Result<i32> {
             .insert(ACCEPT_ENCODING, HeaderValue::from_static("identity"));
     }
 
+    log::trace!("Built reqwest request");
+    // Note: Debug impl is incomplete?
+    log::trace!("{request:#?}");
+
     let buffer = Buffer::new(
         args.download,
         args.output.as_deref(),
@@ -497,7 +514,7 @@ fn run(args: Cli) -> Result<i32> {
             args.headers,
             args.body,
             args.meta,
-            args.quiet,
+            args.quiet > 0,
             args.offline,
             &buffer,
         ),
@@ -565,11 +582,14 @@ fn run(args: Cli) -> Result<i32> {
                 400..=499 => 4,
                 500..=599 => 5,
                 _ => 0,
+            };
+            // Print this if the status code isn't otherwise ending up in the terminal.
+            // HTTPie looks at --quiet, since --quiet always suppresses the response
+            // headers even if you pass --print=h. But --print takes precedence for us.
+            if exit_code != 0 && (is_output_redirected || !print.response_headers) {
+                log::warn!("HTTP {} {}", status.as_u16(), reason_phrase(&response));
             }
         }
-        if is_output_redirected && exit_code != 0 {
-            warn(&format!("HTTP {}", status));
-        }
 
         if print.response_headers {
             printer.print_response_headers(&response)?;
@@ -582,7 +602,7 @@ fn run(args: Cli) -> Result<i32> {
                     &url,
                     resume,
                     pretty.color(),
-                    args.quiet,
+                    args.quiet > 0,
                 )?;
             }
         } else {
@@ -607,3 +627,28 @@ fn run(args: Cli) -> Result<i32> {
 
     Ok(exit_code)
 }
+
+/// Configure backtraces for standard panics and anyhow using `$RUST_BACKTRACE`.
+///
+/// Note: they only check the environment variable once, so this won't take effect if
+/// we do it after a panic has already happened or an anyhow error has already been
+/// created.
+///
+/// It's possible for CLI parsing to create anyhow errors before we call this function
+/// but it looks like those errors are always fatal.
+///
+/// https://github.com/rust-lang/rust/issues/93346 will become the preferred way to
+/// configure panic backtraces.
+fn setup_backtraces() {
+    if std::env::var_os("RUST_BACKTRACE").is_some() {
+        // User knows best
+        return;
+    }
+
+    // SAFETY: No other threads are running at this time.
+    // (Will become unsafe in the 2024 edition.)
+    #[allow(unused_unsafe)]
+    unsafe {
+        std::env::set_var("RUST_BACKTRACE", "1");
+    }
+}
